package xql;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.logging.Logger;

/**
 * $Log: $
 */
public class DbAccess {
    private static final Logger log = Logger.getLogger(DbAccess.class.getName());

    private String driver=null,url=null,login=null,pass=null;

    public DbAccess(Presets.Preset ps) {
        configureFromPreset(ps);
    }

    public DbAccess() {
    }

    public void setDriver(String driver) {
        this.driver = driver;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public void setPass(String pass) {
        this.pass = pass;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public void configureFromPreset(Presets.Preset ps){
        driver=ps.getDriver();
        url=ps.getUrl();
        login=ps.getLogin();
        pass=ps.getPass();
    }
    private boolean isNull(String driver) {
        return driver==null|| "".equals(driver.trim());
    }
    
    public void emitWarnings(){
        if(isNull(driver)){
            log.warning("no driver specified! either use the "+Main.jdbcdriver.getArgName()+
                    " option or use a preset which has a driver value");
        }
        if(isNull(url)){
            log.warning("no url specified! either use the "+Main.jdbcurl.getArgName()+
                    " option or use a preset which has a url value");
        }
        if(isNull(login)){
            log.warning("no login specified! either use the "+Main.dbusername.getArgName()+
                    " option or use a preset which has a login value");
        }
        if(isNull(pass)){
            log.warning("no pass specified! either use the "+Main.dbpassword.getArgName()+
                    " option or use a preset which has a pass value");
        }
    }

    public void savePreset(Presets p, String name) throws IOException {
        p.save(name, driver,login, pass,url);
    }

    public Connection openConnection() throws Exception {
        try {
            log.info("loading jdbc driver="+driver);
            Class.forName(driver);
        } catch (Exception e) {
            System.err.println("couldn't load driverclass="+driver);
            throw e;
        }
        log.info("getting connection url="+url+" name="+login);
        return DriverManager.getConnection(url,login,pass);
    }
}
