/*    xql JDBC SQL client
    Copyright (C) 2004  Nathaniel Spurling

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package xql;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.File;
import java.util.*;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;


public class ExtraClassLoader extends ClassLoader {
    private static final Logger log = Logger.getLogger(ExtraClassLoader.class.getName());

    private List classpathFiles;
    public ExtraClassLoader(ClassLoader parent, List<File> classpathFiles) {
        super(parent);
        this.classpathFiles = classpathFiles;
        if(classpathFiles.size()==0) log.warning("classpathFiles was empty but specified!");
    }

    public String toString() {
        return getClass().getName();
    }

    protected Class loadClass(String name, boolean resolve) throws ClassNotFoundException{
     //   System.out.println("loadclass name="+name);
        if(isLocalClass(name)){//"xql.Executor".equals(name)){
            Class c= findClass(name);
            if(resolve) this.resolveClass(c);
            return c;
        }
        return super.loadClass(name,resolve);
    }

    private boolean isLocalClass(String name) {
        return name.startsWith("xql.") && !"xql.Main".equals(name);
    }

    private Map<String,Class> classes = new HashMap<String,Class>();
    protected Class findClass(String name) throws ClassNotFoundException {
      //  System.out.println("findClass name = " + name);
        if(classes.containsKey(name)) return classes.get(name);
        String resourceName = name.replace('.','/')+".class";
        URL url;
        if(isLocalClass(name)){//"xql.Executor".equals(name)){
            url = getParent().getResource(resourceName);//getParent().findResource(name);
        }
        else url = getUrl(resourceName);

        if(url==null) throw new ClassNotFoundException(resourceName);
        //load up this here class form the url
        try {
            InputStream is = url.openStream();
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int i;
            while((i=is.read())!=-1){
                baos.write(i);
            }
            is.close();
            byte[] bytes = baos.toByteArray();
            baos.close();
            Class c = defineClass(name,bytes,0,bytes.length);
            classes.put(name,c);
         //   System.out.println("sucessfully loaded class="+name);
            return c;
        } catch (IOException e) {
            e.printStackTrace();
            throw new ClassNotFoundException("io exception");
        }
    }

    Map<String,URL> jarIndex = new HashMap<String,URL>();
    private URL getUrl(String name){
        //let's have a scout round some of those old jars and zips; see if we can dig out some bytes for ya
        if(!jarsIndexed) indexJarsAndFiles();
        return jarIndex.get(name);
    }
    private void indexJarsAndFiles(){
        for (Iterator iterator1 = classpathFiles.iterator(); iterator1.hasNext();) {
            File f = (File)iterator1.next();
            if(!f.exists()) continue;
            if(f.getName().endsWith(".jar") || f.getName().endsWith(".zip")){
                //got ourselves a zip file, presumably containing classes
                log.fine("indexing jar="+f);
                try {
                    ZipFile zf = new ZipFile(f);
                    Enumeration en = zf.entries();
                    while (en.hasMoreElements()) {
                        ZipEntry ze = (ZipEntry) en.nextElement();
                        String zeName = ze.getName();
                        String jarUrl = "jar:"+f.toURL()+"!/"+zeName;
                        try{
                            URL value = new URL(jarUrl);
                            jarIndex.put(zeName,value);
                        } catch (MalformedURLException e) {
                            log.log(Level.WARNING,"couldn't index file="+f+" url="+jarUrl);
                        }
                    }
                } catch (IOException e) {
                    log.log(Level.WARNING,"couldn't index jar="+f);
                }
            }
            else{
                try {
                    jarIndex.put(f.getName(),f.toURL());
                } catch (MalformedURLException e) {
                    log.log(Level.WARNING,"couldn't index file="+f);
                }
            }
        }
        if(log.isLoggable(Level.FINEST)){
            log.finest("jarindex="+jarIndex);
        }
        jarsIndexed=true;
    }
    private boolean jarsIndexed=false;
}
