/*    xql JDBC SQL client
    Copyright (C) 2004  Nathaniel Spurling

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package xql;

//import org.apache.commons.cli.*;

import uk.co.hcsoftware.yago.ArgReader;
import uk.co.hcsoftware.yago.Argument;
import uk.co.hcsoftware.yago.InvalidArgException;
import uk.co.hcsoftware.yago.Option;
import uk.co.hcsoftware.yago.util.DefaultParsers;
import uk.co.hcsoftware.yago.parsers.GnuParser;

import java.io.File;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


public class Main {
    static Logger log = Logger.getLogger(Main.class.getName());
    private static ArgReader<String> stringReader = DefaultParsers.createStringReader();
   
    private static ArgReader<List<File>> createFileListReader(final boolean checkExists){
        return new ArgReader<List<File>>() {
            public List<File> readArg(String argValue) throws InvalidArgException {
                // /var/logs/log.txt:/var/logs/logs2.txt 
                String[] paths=argValue.split(File.pathSeparator);
                List<File> l=new ArrayList<File>(paths.length);
                for(String p:paths){
                    File f = new File(p);
                    if(checkExists && !f.exists()){
                        throw new InvalidArgException("file="+f+" doesn't exist",argValue);
                    }
                    l.add(f);
                }
                return l;
            }
        } ;
    }

    public static final Option<List<File>> classpath =
            new Option<List<File>>("cp",false,createFileListReader(false),"classpath to search for db drivers");
            //getOption("cp","classpath",true,"extra classpath (this will be searched for db drivers)");
    public static final Option<String> preset =
                    new Option<String>("p",false,stringReader,"saved preset you wish to execute against");
                    //getOption("p","preset",true,"saved preset you wish to execute against");
    public static final Option listpresets =
                            new Option("listpresets",false,'l',"listpresets",null,"list presets available and exit");

    public static final Option<File> presetfile
            =new Option<File>("presetfile",false,DefaultParsers.createFileReader(true),"alternative file for presets, default is ${user.home}/.xql/presets.properties");
    //getOption("pf","presetfile",true,"alternative file for presets, default is ${user.home}/.xql/presets.properties");
    public static final Option<String> savetopreset
            =new Option<String>("savetopreset",false,stringReader,"if execution is successful save user,pass,url,driver values to presetfile");

    public static final Option<String> dbusername =
            new Option<String>("user",false,stringReader,"database username, used in preference to value in presets file if specified");
    //getLongOption("login",true,"database username");
    public static final Option<String> dbpassword =
            new Option<String>("pass",false,stringReader,"database password, used in preference to value in presets file if specified");
   // public static final Option dbpassword = getLongOption("password",true,"database password");
     public static final Option<String> jdbcurl =
           new Option<String>("url",false,stringReader,"jdbc url");
   // public static final Option jdbcurl = getOption("url","url",true,"jdbc url");
    public static final Option<String> jdbcdriver =
           new Option<String>("driver",false,stringReader,"database driver e.g. com.foo.db.JdbcDriver");
           //getLongOption("driver",true,"database driver e.g. com.sybase.jdbc2.jdbc.SybDriver");

    public static final Option<Integer> maxrows = new Option<Integer>("maxrows",false
            ,'m',"maxrows"
            ,DefaultParsers.createIntReader(),"max rows to retrieve, default is 50, -1 means unlimited)");
            //getOption("m","maxrows",true,"max rows to retrieve, default is 50, -1 means unlimited)");
    public static final Option updatequery =
                    new Option("u",false,"execute update query which returns no results");
                    //getOption("u","update",false,"execute update query which returns no results");
    public static final Option printtime =
                            new Option("t",false,"print time elapsed between statment.execute and stmt.return");
                            //getOption("t","time",false,"print time taken to between statment.execute and return");
    public static final Option transposeprintout = 
          new Option("x",false,"transpose resultset, i.e. print one cell per line in colname: value format");                          
                     //               getShortOption("x",false,"transpose resultset, i.e. print one cell per line in colname: value format");
    public static final Option dontprintrownums =
          new Option("norownums",false,"don't print row numbers in output");
                             //getLongOption("nr",false,"don't print row numbers in output");
    public static final Option<List<File>> scriptfile =
         new Option<List<File>>("scriptfile",false,createFileListReader(true),File.pathSeparator+" delimited filename(s): execute every line of each file as an sql statement");
                                     //getLongOption("scriptfile",false,"argument(s) are filename(s): execute every line as an sql statement");
    public static final Option<List<File>> sqlfile =
         new Option<List<File>>("sqlfile",false,createFileListReader(true),File.pathSeparator+" delimited filename(s): execute complete content of each file as an sql statement");
    public static final Argument<String> sql =
            new Argument<String>(false,stringReader,"sql","SQL statement to execute");

    public static final Option force =
            new Option("force",false,"Continue executing statements even if exceptions");


    public static final Option<String> copytable =
               new Option<String>("copytable",false,stringReader,"copy table from one database to another. Specify fromPreset:toPreset:tablename:excludedcolumns" +
                       ". e.g. xql --copytable db1:db2:mytable 'select * from mytable': copies mytable from db1 to db2");


    public static Option<Format> format = new Option<Format>("format"
             ,false
             , new FormatArgReader()
            ,"output format, one of: "+Format.printValues());


    public static final Option help = new Option("help",false,"Print usage");

    Option[] opts = {classpath,preset,listpresets,presetfile,dbusername,dbpassword
                     ,jdbcurl,jdbcdriver,maxrows,updatequery,printtime,transposeprintout
                     , dontprintrownums, scriptfile,sqlfile,help,savetopreset,copytable,force,format
    };

    GnuParser gp = new GnuParser(opts,new Argument[]{sql});

    public static void main(String args[]){
        new Main().run(args);
    }

    private void run(String[] args) {
        try{
            if(args.length==1 && "-l".equals(args[0]) || "--listpresets".equals(args[0])){
//                new Presets(getPresetFile()).list(System.out);
                new Presets(Presets.getPresetFile()).list(System.out);
                return;
            }
          //  gp.createAtLeastOneOptionGroup(scriptfile,sqlfile,sql);
//            gp.createBundleOptionGroup(presetfile,listpresets);
//            gp.createBundleOptionGroup(presetfile,preset);
//            gp.createBundleOptionGroup(presetfile,savetopreset);

            gp.parseOrDie("xql",args);
            log.info("XQL version 0.24");
            if(log.isLoggable(Level.INFO)){
                StringBuffer sb = new StringBuffer();
                for(Option o:gp.getSetOptions()){
                    if(sb.length()>0) sb.append(" ");
                    sb.append(o.getName()+"="+o.getArgValue());
                }
                log.fine("options: "+sb.toString());
                StringBuffer sb1 = new StringBuffer();
                for(Argument o:gp.getFoundArguments()){
                    if(sb1.length()>0) sb.append(" ");
                    sb1.append(o.getName()+"="+o.getArgValue());
                }
                log.fine("args: "+sb1.toString());
            }
            if((!Main.scriptfile.isSet() && !Main.sql.wasFound() && !Main.sqlfile.isSet()) || help.isSet()){
                System.out.println(gp.getUsage("xql"));
                return;
            }

            ClassLoader cl;
            if(classpath.isSet()){ //classpath
                log.fine("loading xql.Executor class");
                cl = new ExtraClassLoader(Main.class.getClassLoader(),classpath.getArgValue());
                cl.loadClass("xql.Executor");
            }
            else{
                cl = Main.class.getClassLoader();
            }

            Class c = cl.loadClass("xql.Executor");
            Object o = c.newInstance();
            if (!copytable.isSet()) {
                Method m = c.getMethod("execute",new Class[0]);
                m.invoke(o,new Object[0]);
            }
            else{
                if(!Main.sql.wasFound()){
                    log.warning("Please specify some SQL to tell me what to copy");
                    return;
                }
                Method m = c.getMethod("copy",String.class, String.class, String.class);
                //copy(String fromPreset, String toPreset, String tableName, List<String> ignoreColumnNames)
                //db1:db2:mytable:pk,changetime
                String[] cargs = copytable.getArgValue().split(":");
                String ps1 = cargs[0];
                String ps2 = cargs[1];
                String table=cargs[2];

                /*List<String> excludes = new ArrayList<String>();
                if(cargs.length>3){
                    String[] extables = cargs[3].split(",");
                    for(String s:extables) excludes.add(s);
                }*/
                m.invoke(o,ps1,ps2,table);
            }
        } catch (Exception e) {
            log.log(Level.SEVERE,"fatal exception",e);
            e.printStackTrace();
        }
    }

}


