/*    xql JDBC SQL client
    Copyright (C) 2004  Nathaniel Spurling

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package xql;

import com.Ostermiller.util.CSVParser;

import java.io.*;
import java.util.Date;
import java.util.Properties;
import java.util.logging.Logger;


public class Presets extends Properties{
    private static final Logger log = Logger.getLogger(Presets.class.getName());
    private File pFile;
    private boolean loaded=false;
    public Presets(File pFile) {
        this.pFile = pFile;
    }

    public Preset getPreset(String name) throws NoLoadPresetsException {
        loadIfNec();
        String val = getProperty(name);
        if(val==null) return null;
        return new Preset(name,val);
    }

    public void list(PrintStream out) {
        loadIfNec();
        super.list(out);    //To change body of overridden methods use File | Settings | File Templates.
    }


    /*
    * Rather than use an anonymous inner class to share common code, this
    * method is duplicated in order to ensure that a non-1.1 compiler can
    * compile this file.
    */
    public void list(PrintWriter out) {
        loadIfNec();
        super.list(out);    //To change body of overridden methods use File | Settings | File Templates.
    }

    private void loadIfNec() throws NoLoadPresetsException {
        try {
            if(!loaded){
                final FileInputStream in = new FileInputStream(pFile);
                super.load(in);
                in.close();
                loaded=true;
            }
        } catch (IOException e) {
            throw new NoLoadPresetsException(e,pFile);
        }
    }

    public void save(String name, String driver, String login, String pass, String url) throws IOException {
        String val = url+","+login+","+pass+","+driver;
        setProperty(name,val);
        final FileOutputStream out = new FileOutputStream(pFile);
        super.store(out,"xql database presets, last modified "+(new Date()) );
        out.close();
    }

    public static File getPresetFile() {
        final File f;
        if (Main.presetfile.isSet()) {
            f = Main.presetfile.getArgValue();
        } else {
            final File parent = new File(System.getProperty("user.home"),".xql");
            f=new File(parent,"presets.properties");
            Executor.log.fine("no presetfile arg specified, using user.home");
        }
        Executor.log.info("preset file="+f);
        return f;
    }


    public static class Preset{
        private String name, url, login, pass, driver;
        public Preset(String name, String val){
            try {
                this.name=name;
//                String[] s = val.split(",");

                String[][] values = CSVParser.parse(
                    new StringReader(val)
                );
                String[] s = values[0];

                url=s[0];
                login=s[1];
                pass=s[2];
                driver=s[3];
            } catch (IOException e) {
                throw new RuntimeException("error reading presets",e);
            }
        }

        public String getName() {
            return name;
        }

        public String getUrl() {
            return url;
        }

        public String getLogin() {
            return login;
        }

        public String getPass() {
            return pass;
        }

        public String getDriver() {
            return driver;
        }

        public String toString() {
            return "name="+name+" driver="+driver+" login="+login;
        }
    }
}
